@@ -12,12 +12,31 @@ class Component(metaclass=ABCMeta):
     components inherits from the Component class. 
 
     """
-    def __init__(self, esM, name, dimension,
-                 hasCapacityVariable, capacityVariableDomain='continuous', capacityPerPlantUnit=1,
-                 hasIsBuiltBinaryVariable=False, bigM=None, locationalEligibility=None,
-                 capacityMin=None, capacityMax=None, partLoadMin=None, sharedPotentialID=None, capacityFix=None, isBuiltFix=None,
-                 investPerCapacity=0, investIfBuilt=0, opexPerCapacity=0, opexIfBuilt=0, QPcostScale=0,
-                 interestRate=0.08, economicLifetime=10, technicalLifetime=None, yearlyFullLoadHoursMin=None, 
+    def __init__(self, 
+                 esM, 
+                 name, 
+                 dimension,
+                 hasCapacityVariable, 
+                 capacityVariableDomain='continuous', 
+                 capacityPerPlantUnit=1,
+                 hasIsBuiltBinaryVariable=False, 
+                 bigM=None, 
+                 locationalEligibility=None,
+                 capacityMin=None, 
+                 capacityMax=None, 
+                 partLoadMin=None, 
+                 sharedPotentialID=None, 
+                 capacityFix=None, 
+                 isBuiltFix=None,
+                 investPerCapacity=0, 
+                 investIfBuilt=0, 
+                 opexPerCapacity=0, 
+                 opexIfBuilt=0, 
+                 QPcostScale=0,
+                 interestRate=0.08, 
+                 economicLifetime=10, 
+                 technicalLifetime=None, 
+                 yearlyFullLoadHoursMin=None, 
                  yearlyFullLoadHoursMax=None):
         """
         Constructor for creating an Component class instance.
@@ -288,14 +307,18 @@ class Component(metaclass=ABCMeta):
         :type yearlyFullLoadHoursMin:
             * None or
             * Float with positive (>=0) value or
-            * Pandas Series with positive (>=0) values.
+            * Pandas Series with positive (>=0) values. The indices of the series have to equal the in the
+              energy system model specified locations (dimension=1dim) or connections between these locations
+              in the format of 'loc1' + '_' + 'loc2' (dimension=2dim).
 
         :param yearlyFullLoadHoursMax: if specified, indicates the maximum yearly full load hours.
             |br| * the default value is None
         :type yearlyFullLoadHoursMax:
             * None or
             * Float with positive (>=0) value or
-            * Pandas Series with positive (>=0) values.
+            * Pandas Series with positive (>=0) values. The indices of the series have to equal the in the
+              energy system model specified locations (dimension=1dim) or connections between these locations
+              in the format of 'loc1' + '_' + 'loc2' (dimension=2dim).
 
         :param modelingClass: to the Component connected modeling class.
             |br| * the default value is ModelingClass
@@ -317,9 +340,6 @@ class Component(metaclass=ABCMeta):
         self.bigM = bigM
         self.partLoadMin = partLoadMin
         
-       
- 
-            
         # Set economic data
         elig = locationalEligibility
         self.investPerCapacity = utils.checkAndSetCostParameter(esM, name, investPerCapacity, dimension, elig)
@@ -345,8 +365,8 @@ class Component(metaclass=ABCMeta):
         utils.checkLocationSpecficDesignInputParams(self, esM)
         
         # Set quadratic capacity bounds and residual cost scale (1-cost scale)
-        self.QPbound = utils.getQPbound(esM, self.capacityMax, self.capacityMin)
-        self.QPcostDev = utils.getQPcostDev(esM, self.QPcostScale)
+        self.QPbound = utils.getQPbound(self.QPcostScale, self.capacityMax, self.capacityMin)
+        self.QPcostDev = utils.getQPcostDev(self.QPcostScale)
 
         #
         # # Variables at optimum (set after optimization)
@@ -934,12 +954,19 @@ class ComponentModel(metaclass=ABCMeta):
         compDict, abbrvName = self.componentsDict, self.abbrvName
         opVar, capVar = getattr(pyM, opVarName + '_' + abbrvName), getattr(pyM, 'cap_' + abbrvName)
         constrSet1 = getattr(pyM, constrSetName + '1_' + abbrvName)
-        factor1 = 1 if isStateOfCharge else esM.hoursPerTimeStep
 
-        def op1(pyM, loc, compName, p, t):
-            factor2 = 1 if factorName is None else getattr(compDict[compName], factorName)
-            return opVar[loc, compName, p, t] <= factor1 * factor2 * capVar[loc, compName]
-        setattr(pyM, constrName + '1_' + abbrvName, pyomo.Constraint(constrSet1, pyM.timeSet, rule=op1))
+        if not pyM.hasSegmentation:
+            factor1 = 1 if isStateOfCharge else esM.hoursPerTimeStep
+            def op1(pyM, loc, compName, p, t):
+                factor2 = 1 if factorName is None else getattr(compDict[compName], factorName)
+                return opVar[loc, compName, p, t] <= factor1 * factor2 * capVar[loc, compName]
+            setattr(pyM, constrName + '1_' + abbrvName, pyomo.Constraint(constrSet1, pyM.timeSet, rule=op1))
+        else:
+            factor1 = (esM.hoursPerSegment/esM.hoursPerSegment).to_dict() if isStateOfCharge else esM.hoursPerSegment.to_dict()
+            def op1(pyM, loc, compName, p, t):
+                factor2 = 1 if factorName is None else getattr(compDict[compName], factorName)
+                return opVar[loc, compName, p, t] <= factor1[p,t] * factor2 * capVar[loc, compName]
+            setattr(pyM, constrName + '1_' + abbrvName, pyomo.Constraint(constrSet1, pyM.timeSet, rule=op1))
 
     def operationMode2(self, pyM, esM, constrName, constrSetName, opVarName, opRateName='operationRateFix',
                        isStateOfCharge=False):
@@ -952,12 +979,19 @@ class ComponentModel(metaclass=ABCMeta):
         compDict, abbrvName = self.componentsDict, self.abbrvName
         opVar, capVar = getattr(pyM, opVarName + '_' + abbrvName), getattr(pyM, 'cap_' + abbrvName)
         constrSet2 = getattr(pyM, constrSetName + '2_' + abbrvName)
-        factor = 1 if isStateOfCharge else esM.hoursPerTimeStep
 
-        def op2(pyM, loc, compName, p, t):
-            rate = getattr(compDict[compName], opRateName)
-            return opVar[loc, compName, p, t] == capVar[loc, compName] * rate[loc][p, t] * factor
-        setattr(pyM, constrName + '2_' + abbrvName, pyomo.Constraint(constrSet2, pyM.timeSet, rule=op2))
+        if not pyM.hasSegmentation:
+            factor = 1 if isStateOfCharge else esM.hoursPerTimeStep
+            def op2(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] == capVar[loc, compName] * rate[loc][p, t] * factor
+            setattr(pyM, constrName + '2_' + abbrvName, pyomo.Constraint(constrSet2, pyM.timeSet, rule=op2))
+        else:
+            factor = (esM.hoursPerSegment/esM.hoursPerSegment).to_dict() if isStateOfCharge else esM.hoursPerSegment.to_dict()
+            def op2(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] == capVar[loc, compName] * rate[loc][p, t] * factor[p,t]
+            setattr(pyM, constrName + '2_' + abbrvName, pyomo.Constraint(constrSet2, pyM.timeSet, rule=op2))
 
     def operationMode3(self, pyM, esM, constrName, constrSetName, opVarName, opRateName='operationRateMax',
                        isStateOfCharge=False):
@@ -970,12 +1004,19 @@ class ComponentModel(metaclass=ABCMeta):
         compDict, abbrvName = self.componentsDict, self.abbrvName
         opVar, capVar = getattr(pyM, opVarName + '_' + abbrvName), getattr(pyM, 'cap_' + abbrvName)
         constrSet3 = getattr(pyM, constrSetName + '3_' + abbrvName)
-        factor = 1 if isStateOfCharge else esM.hoursPerTimeStep
 
-        def op3(pyM, loc, compName, p, t):
-            rate = getattr(compDict[compName], opRateName)
-            return opVar[loc, compName, p, t] <= capVar[loc, compName] * rate[loc][p, t] * factor
-        setattr(pyM, constrName + '3_' + abbrvName, pyomo.Constraint(constrSet3, pyM.timeSet, rule=op3))
+        if not pyM.hasSegmentation:
+            factor = 1 if isStateOfCharge else esM.hoursPerTimeStep
+            def op3(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] <= capVar[loc, compName] * rate[loc][p, t] * factor
+            setattr(pyM, constrName + '3_' + abbrvName, pyomo.Constraint(constrSet3, pyM.timeSet, rule=op3))
+        else:
+            factor = (esM.hoursPerSegment/esM.hoursPerSegment).to_dict() if isStateOfCharge else esM.hoursPerSegment.to_dict()
+            def op3(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] <= capVar[loc, compName] * rate[loc][p, t] * factor[p,t]
+            setattr(pyM, constrName + '3_' + abbrvName, pyomo.Constraint(constrSet3, pyM.timeSet, rule=op3))
 
     def operationMode4(self, pyM, esM, constrName, constrSetName, opVarName, opRateName='operationRateFix'):
         """
@@ -985,10 +1026,16 @@ class ComponentModel(metaclass=ABCMeta):
         opVar = getattr(pyM, opVarName + '_' + abbrvName)
         constrSet4 = getattr(pyM, constrSetName + '4_' + abbrvName)
 
-        def op4(pyM, loc, compName, p, t):
-            rate = getattr(compDict[compName], opRateName)
-            return opVar[loc, compName, p, t] == rate[loc][p, t]
-        setattr(pyM, constrName + '4_' + abbrvName, pyomo.Constraint(constrSet4, pyM.timeSet, rule=op4))
+        if not pyM.hasSegmentation:
+            def op4(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] == rate[loc][p, t]
+            setattr(pyM, constrName + '4_' + abbrvName, pyomo.Constraint(constrSet4, pyM.timeSet, rule=op4))
+        else:
+            def op4(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] == rate[loc][p, t] * esM.timeStepsPerSegment.to_dict()[p, t]
+            setattr(pyM, constrName + '4_' + abbrvName, pyomo.Constraint(constrSet4, pyM.timeSet, rule=op4))
 
     def operationMode5(self, pyM, esM, constrName, constrSetName, opVarName, opRateName='operationRateMax'):
         """
@@ -998,10 +1045,16 @@ class ComponentModel(metaclass=ABCMeta):
         opVar = getattr(pyM, opVarName + '_' + abbrvName)
         constrSet5 = getattr(pyM, constrSetName + '5_' + abbrvName)
 
-        def op5(pyM, loc, compName, p, t):
-            rate = getattr(compDict[compName], opRateName)
-            return opVar[loc, compName, p, t] <= rate[loc][p, t]
-        setattr(pyM, constrName + '5_' + abbrvName, pyomo.Constraint(constrSet5, pyM.timeSet, rule=op5))
+        if not pyM.hasSegmentation:
+            def op5(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] <= rate[loc][p, t]
+            setattr(pyM, constrName + '5_' + abbrvName, pyomo.Constraint(constrSet5, pyM.timeSet, rule=op5))
+        else:
+            def op5(pyM, loc, compName, p, t):
+                rate = getattr(compDict[compName], opRateName)
+                return opVar[loc, compName, p, t] <= rate[loc][p, t] * esM.timeStepsPerSegment.to_dict()[p, t]
+            setattr(pyM, constrName + '5_' + abbrvName, pyomo.Constraint(constrSet5, pyM.timeSet, rule=op5))
 
 
     def additionalMinPartLoad(self, pyM, esM, constrName, constrSetName, opVarName, opVarBinName, capVarName):
@@ -1033,7 +1086,6 @@ class ComponentModel(metaclass=ABCMeta):
 
     def yearlyFullLoadHoursMin(self, pyM, esM):
         # TODO: Add deprecation warning to sourceSink.yearlyLimitConstraint and call this function in it
-        # TODO: Use a set to declare Constraint only for components with full load hour limit set.
         """
         Limit the annual full load hours to a minimum value.
 
@@ -1048,15 +1100,14 @@ class ComponentModel(metaclass=ABCMeta):
         capVar = getattr(pyM, 'cap_' + abbrvName)
         yearlyFullLoadHoursMinSet = getattr(pyM, 'yearlyFullLoadHoursMinSet_' + abbrvName)
 
-
-        def yearlyFullLoadHoursMinConstraint(pyM, loc, compName, p, t):
-            full_load_hours = sum(
-                opVar[loc, compName, p, t] * esM.periodOccurrences[p] / esM.numberOfYears for loc, compName, p, t,
-                in opVar)
+        def yearlyFullLoadHoursMinConstraint(pyM, loc, compName):
+            full_load_hours = (
+                sum(opVar[loc, compName, p, t] * esM.periodOccurrences[p] for p, t in pyM.timeSet) / esM.numberOfYears
+            )
             return full_load_hours >= capVar[loc, compName] * compDict[compName].yearlyFullLoadHoursMin[loc]
 
         setattr(pyM, 'ConstrYearlyFullLoadHoursMin_' + abbrvName,
-                pyomo.Constraint(yearlyFullLoadHoursMinSet, pyM.timeSet, rule=yearlyFullLoadHoursMinConstraint))
+                pyomo.Constraint(yearlyFullLoadHoursMinSet, rule=yearlyFullLoadHoursMinConstraint))
 
     def yearlyFullLoadHoursMax(self, pyM, esM):
         """
@@ -1073,14 +1124,14 @@ class ComponentModel(metaclass=ABCMeta):
         capVar = getattr(pyM, 'cap_' + abbrvName)
         yearlyFullLoadHoursMaxSet = getattr(pyM, 'yearlyFullLoadHoursMaxSet_' + abbrvName)
 
-        def yearlyFullLoadHoursMaxConstraint(pyM, loc, compName, p, t):
-            full_load_hours = sum(
-                opVar[loc, compName, p, t] * esM.periodOccurrences[p] / esM.numberOfYears for loc, compName, p, t,
-                in opVar)
+        def yearlyFullLoadHoursMaxConstraint(pyM, loc, compName):
+            full_load_hours = (
+                sum(opVar[loc, compName, p, t] * esM.periodOccurrences[p] for p, t in pyM.timeSet) / esM.numberOfYears
+            )
             return full_load_hours <= capVar[loc, compName] * compDict[compName].yearlyFullLoadHoursMax[loc]
 
         setattr(pyM, 'ConstrYearlyFullLoadHoursMax_' + abbrvName,
-                pyomo.Constraint(yearlyFullLoadHoursMaxSet, pyM.timeSet, rule=yearlyFullLoadHoursMaxConstraint))
+                pyomo.Constraint(yearlyFullLoadHoursMaxSet, rule=yearlyFullLoadHoursMaxConstraint))
 
 
     ####################################################################################################################
